<?php

namespace Iwaydev\TransNgClient;

use \DomainException;
use Iwaydev\BaseSender\Sender;
use JsonException;
use Psr\Http\Message\ResponseInterface;

/**
 * Class TransNgClient
 *
 * @package Iwaydev\TransNgClient
 */
class TransNgClient extends Sender
{
    /**
     * Авторизация
     *
     * @param string $token Токен авторизации
     *
     * @return $this
     */
    public function auth(string $token): self
    {
        $this->setHeader('Authorization', 'Bearer ' . $token);

        return $this;
    }

    /**
     * Авторизация
     *
     * @param string $login
     * @param string $password
     *
     * @return string
     * @throws JsonException
     */
    public function login(string $login, string $password): string
    {
        $response = $this->post('auth/login', [
            'login' => $login,
            'password' => $password,
        ]);

        return $response['token'];
    }


    /**
     * Возвращает прайсы согласно переданным фильтрам
     *
     * @param array $query Фильтры
     *
     * @return array
     * @throws JsonException
     */
    public function getPrices(array $query = []): array
    {
        return $this->get('prices', $query);
    }


    /**
     * Возвращает поездки согласно переданным фильтрам
     *
     * @param array $query Фильтры
     *
     * @return array
     * @throws JsonException
     */
    public function getTrips(array $query = []): array
    {
        return $this->get('orders/trips', $query);
    }

    /**
     * Создание поездок
     *
     * @param array $data
     *
     * @return array
     * @throws JsonException
     */
    public function createTrips(array $data): array
    {
        return $this->post('orders', $data);
    }

    /**
     * Редактирование поездок
     *
     * @param array $data
     *
     * @return array
     * @throws JsonException
     */
    public function updateTrips(array $data): array
    {
        return $this->put('orders/trips', $data);
    }

    /**
     * Одобрить поездку
     *
     * @param int $transaction
     * @param array $body
     *
     * @return array
     * @throws JsonException
     */
    public function approveTrip(int $transaction, array $body = []): array
    {
        return $this->post('orders/approve/' . $transaction, $body);
    }

    /**
     * Проверка отмены поездки
     *
     * @param array $data
     *
     * @return array
     * @throws JsonException
     */
    public function checkCancelTrip(int $tripId, array $data): array
    {
        return $this->get('orders/' . $tripId . '/check-cancel', $data);
    }

    /**
     * Отменить поездку
     *
     * @param array $data
     *
     * @return array
     * @throws JsonException
     */
    public function cancelTrip(array $data): array
    {
        return $this->post('orders/trips/cancel', $data);
    }


    /**
     * Метод для получения настроек пользователя
     *
     * @param array $data
     *
     * @return array
     * @throws JsonException
     */
    public function getSettingUser(array $data = []): array
    {
        return $this->get('partners/get-setting-user', $data);
    }

    /**
     * Метод для получения транспортных узлов по названию
     *
     * @param array $query Данные запроса
     *
     * @return array
     * @throws JsonException
     */
    public function getTransportNodes(array $query = []): array
    {
        return $this->get('transport-nodes', $query);
    }

    /**
     * Метод отправки ваучера по заказу
     *
     * @param int $transaction
     * @param array $data
     *
     * @return array
     * @throws JsonException
     */
    public function documentsSend(int $transaction, array $data = []): array
    {
        return $this->post('documents/send/' . $transaction, $data);
    }

    /**
     * Метод страны по координате
     *
     * @param string $point Координата
     *
     * @return array
     * @throws JsonException
     */
    public function getCountryByPoint(string $point): array
    {
        return $this->get('places/get-country-by-point', ['point' => $point]);
    }

    /**
     * Получение статистики по основным компонентам проекта
     *
     * @param array $data
     *
     * @return array
     * @throws JsonException
     */
    public function monitor(array $data = []): array
    {
        return $this->get('monitor', $data);
    }


    /**
     * Получение последних координат водителя
     *
     * @param array $query Фильтры
     *
     * @return array
     * @throws JsonException
     */
    public function getLastLocationDriver(array $query = []): array
    {
        return $this->get('orders/last-location-driver', $query);
    }

    /**
     * Получение списка последних координат водителей по массиву идентификаторов поездок
     *
     * @param array $query Фильтры
     *
     * @return array
     * @throws JsonException
     */
    public function getLastLocationsDriver(array $query = []): array
    {
        return $this->get('orders/last-locations-driver', $query);
    }


    /**
     * @inheritDoc
     */
    protected function prepareResponse(ResponseInterface $response): array
    {
        $responseData = parent::prepareResponse($response);

        if (isset($responseData['result'])) {
            return $responseData['result'];
        }

        $message = reset($responseData['error']);
        $message = is_array($message) ? $message : $responseData['error'];
        $code = (isset($message['code'])) ? $message['code'] : 0;
        $message = $message['message'] ?? implode(' ', $message);

        throw new DomainException($message, $code);
    }
}
